# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import netsvc
from osv import fields, osv
import time
from mx import DateTime

class payment_order(osv.osv):
    _inherit='payment.order'
    _columns={
        'inter_bank_journal': fields.many2one('account.journal', 'Intermediate Bank Journal', required=False),
    }
payment_order()

class account_invoice(osv.osv):
    _inherit='account.invoice'
    _columns={
        'next_payment_date': fields.date('Next Payment Date', states={'open': [('readonly', True)], 'close': [('readonly', True)]},),
    }
    
    def action_date_assign(self, cr, uid, ids, *args):
        data = super(account_invoice, self).action_date_assign(cr, uid, ids, *args)
        for inv in self.browse(cr, uid, ids):
            if inv.payment_term.id:
                payment_data = self.pool.get('account.payment.term').browse(cr, uid, inv.payment_term.id).cash_discount_ids
                if payment_data:
                    delay = payment_data[0].delay
                    if inv.date_invoice:
                        date_invoice = inv.date_invoice
                    else:
                        date_invoice = time.strftime('%Y-%m-%d')
                    self.write(cr, uid, [inv.id], {'next_payment_date': (DateTime.strptime(date_invoice, '%Y-%m-%d') + DateTime.RelativeDateTime(days = delay)).strftime('%Y-%m-%d')})
        return data
        
    def _next_date(self, cr, uid, ids, context=None):
        payment_term_obj = self.pool.get('account.payment.term')
        current_date = time.strftime('%Y-%m-%d')
        ids = self.search(cr, uid, [('state', '=', 'open')])
        for id in ids:
            invoice_data = self.browse(cr, uid, id, context=context)
            next_pay_date = invoice_data.next_payment_date
            if invoice_data.payment_term:
                payment_data = payment_term_obj.browse(cr, uid, invoice_data.payment_term.id)
            if next_pay_date: 
                if next_pay_date < current_date:
                    for dis_line in payment_data.cash_discount_ids:
                        discount_date = (DateTime.strptime(invoice_data.date_invoice, '%Y-%m-%d') + DateTime.RelativeDateTime(days=dis_line.delay)).strftime('%Y-%m-%d')
                        if discount_date > current_date:
                            self.write(cr, uid, [id], {'next_payment_date': discount_date})
                            break
        return True
    
    def check_next_date(self, cr, uid, ids=[], context=None):
        self._next_date(cr, uid, ids, context=context)
        
    def copy(self, cr, uid, id, default=None, context=None):
        if default is None:
            default = {}
        default = default.copy()
        default.update({'next_payment_date': False})
        return super(account_invoice, self).copy(cr, uid, id, default, context)

account_invoice()

class payment_line(osv.osv):
    _name = 'payment.line'
    _inherit = 'payment.line'
    _columns = {
        'cash_discount': fields.float('Cash Discount'),
        'discount_date': fields.date('Discount Date'),
        'pay_amount': fields.float('Pay Amount'),
    }
payment_line()

class res_partner(osv.osv):
    _inherit = 'res.partner'
    _columns = {
        'property_customer_cash_discount_account': fields.property(
            'account.account',
            type = 'many2one',
            relation = 'account.account',
            string = "Customer Cash Discount Account",
            method = True,
            view_load = True,
            required = True),
        'property_supplier_cash_discount_account': fields.property(
            'account.account',
            type = 'many2one',
            relation = 'account.account',
            string = "Supplier Cash Discount Account",
            method = True,
            view_load = True,
            required = True),
      }
res_partner()
